/*
 * CGI clock
 * Copyright (C) 1999 Open World Ltd
 * Written by Shevek <shevek@anarres.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include "library.h"
#include "hash.h"
#include "rope.h"
#include "cgi.h"
#include "smalloc.h"
#include "log.h"

/* Define DIR_ZONES if you want to use a non-system default set
 * of zoneinfo files, e.g. to install a copy of the Linux
 * zoneinfo files on a Solaris system. */
/* #define DIR_ZONES		DIR_DATA "/" DEFAULT "/clock/zoneinfo/" */

/* Fingers out from here on */
#define STREQ(x, y) (strcasecmp(x, y) == 0)

typedef
struct {
	char	*tz;
	char	*digitset;
	int		 digit;
} clockinfo_t;

void
clock_show(clockinfo_t *info)
{
	time_t		 timebuf;
	struct tm	*tm;
	int			 val;
	char		*server;
	char		*env;
	int			 len;

	server = getenv(ENV_HOSTNAME);
	if (!server) {
		log("cgiconf", PRI_CRIT, "No \"" ENV_HOSTNAME
						"\" in environment");
		exit(0);
	}

	len = 4 + strlen(info->tz);
#ifdef DIR_ZONES
	len += strlen(DIR_ZONES);
#endif
	env = smalloc(len);
#ifdef DIR_ZONES
	sprintf(env, "TZ=" DIR_ZONES "%s", info->tz);
#else
	sprintf(env, "TZ=%s", info->tz);
#endif
	putenv(env);
	tzset();
	time(&timebuf);
	tm = localtime(&timebuf);

	switch(info->digit) {
		case 0: val = tm->tm_min % 10; break;	/* Minute units */
		case 1: val = tm->tm_min / 10; break;	/* Minute tens */
		case 2: val = tm->tm_hour % 10; break;	/* Hour units */
		case 3: val = tm->tm_hour / 10; break;	/* Hour tens */
		default:	val = 0;	/* Defeat compile warnings */
	}

	printf("Location: http://%s" DIR_RESOURCE "/clock/%s/digits/%c.gif"
					"\n\n",
					server, info->digitset, ((char)(val + '0')) );
}


/* QUERY_STRING format: Traditional CGI a=b&c=d
 * Parameters:
 * location			- Unix timezone to use
 * digitPosition	- digit to show, may be (hours|minutes)(tens|units)
 * All other parameters are ignored
 */
void
clock_info_long(clockinfo_t *info)
{
	hash_t	*cgihash;
	char	*temp;

	cgihash = cgi_hash_new();

/* Get the timezone */
	info->tz = hash_get(cgihash, "location");
	if (!info->tz) {
		log("cgiconf", PRI_CRIT, "No 'location' in QUERY_STRING");
		exit(0);
	}

/* Get the digit set */
	info->digitset = hash_get(cgihash, "case");
	if (!info->digitset)
		info->digitset = "default";

/* Get the digit */
	temp = hash_get(cgihash, "digitPosition");
	if (!temp) {
		log("cgiconf", PRI_CRIT, "No 'digitPosition' in QUERY_STRING");
		exit(0);
	}
	if (strlen(temp) == 1)
		info->digit = ((unsigned char)(*temp++)) % 4;
	else if (STREQ(temp, "hourstens"))
		info->digit = 3;
	else if (STREQ(temp, "hoursunits"))
		info->digit = 2;
	else if (STREQ(temp, "minutestens"))
		info->digit = 1;
	else if (STREQ(temp, "minutesunits"))
		info->digit = 0;
	else {
		log("cgiconf", PRI_CRIT, "digitPosition parameter '%s' is "
						"invalid", temp);
		exit(0);
	}
}

int
main(void)
{
	clockinfo_t	 ci;

	library_init("clock");
	clock_info_long(&ci);
	clock_show(&ci);

	return 0;
}
