%{
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include "library.h"
#include "cgi.h"
#include "rope.h"
#include "smalloc.h"
#include "vector.h"
#include "utils.h"
#include "log.h"

#define TEMPLATE(x) ((template_t *)(x))

typedef
struct _template_t {
	char	*path;		/* Filename of template */
	void	*results;	/* Results vector - quadratic */
	void	*resultbuf;	/* Extensible string for results */
} template_t;

// template_t	*yytpt;		/* Current template being parsed */
vector_t	*yyresults;	/* Results vector - quadratic */
vector_t	*yyvec;		/* Results vector - current row */
rope_t		*resultbuf;	/* Rope for repeated section */
FILE		*yystream;	/* Input stream from template */

#if 0
static inline void template_field_print(char *str);
static inline bool template_field_null(char *str);
#endif

#define template_field_null(str) (!!strlen(template_field((str))))
#define template_field_print(str) printf("%s", template_field((str)))

static inline char *template_field(char *str);
static void template_results_buffer(char *str);
static void template_results_print(void);

%}

%option nounput
%option noyywrap

	/* Discard all input */
%x DISCARD
	/* Building results buffer */
%x BUFFER
	/* Printing results buffer N times */
%x RES
	/* Discard parts of loop results buffer */
%x RDIS

	/* Start of results block */
tRESULTS	\{Results\}
	/* End of results block */
tERESULTS	\{\/Results\}
	/* Start of no results block */
tNORESULTS	\{NoResults\}
	/* End of no results block */
tENORESULTS	\{\/NoResults\}

tDIGIT		[0-9]
tNUMBER		{tDIGIT}+

	/* Start of null field block */
tFIELD_NULL		\{{tNUMBER}Null\}
	/* End of null field block */
tEFIELD_NULL	\{\/{tNUMBER}Null\}

	/* Start of not null field block */
tFIELD_NONULL	\{{tNUMBER}NotNull\}
	/* End of not null field block */
tEFIELD_NONULL	\{\/{tNUMBER}NotNull\}


	/* Data */
tFIELD_DATA		\{{tNUMBER}\}

%%

	/* Called once if blank results vector */
<INITIAL>{tNORESULTS}	{	if (vector_length(yyresults)) {
								BEGIN(DISCARD);
							}
						}

	/* Build the string for the results loop */
<INITIAL>{tRESULTS}		{	if (vector_length(yyresults))
								BEGIN(BUFFER);
							else
								BEGIN(DISCARD);
						}

<INITIAL>{tENORESULTS}	{										}

<BUFFER>{tERESULTS}		{	template_results_print();
							BEGIN(INITIAL);
						}
<BUFFER>[^\{]+			{	template_results_buffer(yytext);	}
<BUFFER>\{				{	template_results_buffer(yytext);	}

<DISCARD>[^\{]+			{										}
<DISCARD>\{				{										}
<DISCARD>{tERESULTS}	{ BEGIN(INITIAL);						}
<DISCARD>{tENORESULTS}	{ BEGIN(INITIAL);						}


	/* Result loop lex rules */
<RES>{tFIELD_DATA}		{	template_field_print(yytext);		}

<RES>{tFIELD_NULL}		{	if (template_field_null(yytext))
								BEGIN(RDIS);
						}

<RES>{tFIELD_NONULL}	{	if (!template_field_null(yytext))
								BEGIN(RDIS);
						}

<RES>[^\{]+				{ ECHO;									}
<RES>\{					{ ECHO;									}
<RES>{tEFIELD_NULL}		{										}
<RES>{tEFIELD_NONULL}	{										}

<RDIS>[^\{]+			{										}
<RDIS>\{				{										}
<RDIS>{tEFIELD_NULL}	{ BEGIN(RES);							}
<RDIS>{tEFIELD_NONULL}	{ BEGIN(RES);							}

%%

static FILE *
template_stream(rope_t *pathstr)
{
	char	*path;
	FILE	*input;

	path = rope_chars(pathstr);

	if (access(path, R_OK) != 0) {
		log("cgiconf", PRI_CRIT, "Could not open template '%s' for "
						"reading: %s", path, strerror(errno));
		exit(0);
	}
	resultbuf = rope_new("");

	input = fopen(path, "r");
	if (!input) {
		log("cgiconf", PRI_CRIT, "Could not open template '%s' for "
						"reading: %s", path, strerror(errno));
		exit(0);
	}

	return input;
}

void	/* template_t, hash_table_t */
template_parse(rope_t *pathstr, vector_t *results)
{
	FILE	*stream;

	yyin = template_stream(pathstr);

	printf("Content-type: text/html\n\n");

	/* Better save this in case flex buffer switching does not
	 * reassign to yyin */
	stream = yyin;

	/* Set the global such that we may access it in rules */
	yyresults = results;

	/* Here we go */
	yylex();

	/* Piss off quick */
	fclose(stream);
}

static void
template_results_buffer(char *str)
{
	rope_add(resultbuf, str);
}

static void
template_results_print(void)
{
	YY_BUFFER_STATE	 buf;
	YY_BUFFER_STATE	 oldbuf;
	int				 i;
	int				 len;

#if 0
	DPRINTF(("Results buffer is: >\n%s\n<\n", rope_chars(resultbuf)));
#endif

	oldbuf = YY_CURRENT_BUFFER;

	len = vector_length(yyresults);
	for (i = 0; i < len; i++) {
		BEGIN(RES);
		yyvec = vector_data(yyresults, i);

		buf = yy_scan_string(rope_chars(resultbuf));
		yylex();
		yy_delete_buffer(buf);
	}
	rope_delete(resultbuf);

	BEGIN(INITIAL);
	yy_switch_to_buffer(oldbuf);
}

static inline int
template_field_offset(char *str)
{
	return atol(++str) - 1;
}

static inline char *
template_field(char *str)
{
	char	*data;
	char	*temp;

	data = (char *)vector_data(yyvec, template_field_offset(str));
	if (!data) {
		temp = implode(yyvec, "|");
		log("cgisys", PRI_ERR, "template_field: Expected data "
						"in yyvec[%d] (%s), vector is %s",
						template_field_offset(str), str, temp);
		sfree(temp);
		data = "(null)";
	}
	return data;
}

#if 0
static inline bool
template_field_null(char *str)
{
	return !!strlen(template_field(str));
}

static inline void
template_field_print(char *str)
{
	printf("%s", template_field(str));
}
#endif
