#ifndef __LOG_H__
#define __LOG_H__

#include <sys/types.h>

#define PRI_DEBUG	(7)
#define PRI_INFO	(6)
#define PRI_NOTICE	(5)
#define PRI_WARNING	(4)
#define PRI_ERR		(3)
#define PRI_CRIT	(2)
#define PRI_ALERT	(1)
#define PRI_EMERG	(0)
/* I can use -1 as I'm not doing bitwise operations in the same way
 * that the kernel does it. However, I reserve the right. */
#define PRI_NONE	(-1)	/* Returned by getpri for '*.none' */
#define PRI_ALL		(-2)	/* Returned by getpri for '*.*' */
#define PRI_UNKNOWN	(-3)	/* Used for indicating errors */

#define PRI_NPRI	(8)	/* Number of priorities */

#define LOG_SITELEN		64
#define LOG_REMOTELEN	64
#define LOG_REFERERLEN	256
#define LOG_URLLEN		256
#define LOG_PROGRAMLEN	32
#define LOG_FACILITYLEN	32
#define LOG_MESSAGELEN	4096

#define NO_SITE		"none"
#define NO_REMOTE	"none"
#define NO_URL		"none"
#define NO_REFERER	"none"
#define NO_PROGRAM	"none"
#define NO_FACILITY	"none"
#define NO_MESSAGE	""

/* Entries marked 'X' will be automatically filled in if NULL.
 * Entries marked 'R' will always be overwritten. */
typedef
struct _errmsg_t {
	int		 version;					/* Protocol version */
	pid_t	 pid;						/* X Something... */
	char	 site[LOG_SITELEN];			/* X Site hostname */
	int		 port;						/* X Site port number */
	char	 remote[LOG_REMOTELEN];		/* Remote host */
	char	 referer[LOG_REFERERLEN];	/* X Referer */
	char	 url[LOG_URLLEN];			/* X URL */
	char	 program[LOG_PROGRAMLEN];	/* R CGI script name */
	char	 facility[LOG_FACILITYLEN];	/*   Log message facility */
	int		 priority;					/*   Log message priority */
	char	 message[LOG_MESSAGELEN];	/*   Log message */
} errmsg_t;

void log(char *facility, int priority, char *format, ...);
int	 log_msg(errmsg_t *msg, char *host, int port);

#endif
