#!/usr/bin/perl -w

# ===================================================================
#                    Edit these if you need to
# ===================================================================

# The directory you want the 'Top/' directory to live in on the web
# server. Must be absolute to root.
$dstdir = "/odp";

# Directories to exclude. These are Perl regular expressions.
@exclude = (
	"Adult",
	"Arts",
	"Business",
	"Computers",
	"Games",
	"Health",
	"Home",
	"News",
	"Reference",
	"Science",
	"Shopping",
	"Society",
	"Sports",
	"Bookmarks",
	"World"
	);

# ===================================================================
#                       Hands off below here
# ===================================================================

@titles = ();		# Title stack, for header bar
$start = "Top";		# Start directory, must be "Top", relative to `pwd`
%pagecount = ();	# Number of real pages under a directory

open(DEBUG, ">-") or die "Can't open DEBUG: $!";

sub parse_content;
sub parse_structure;

sub page_count_build {
	my $dir = shift;
	my $count;
	my @dirs;

	return $pagecount{$dir} if (defined($pagecount{$dir}));

	if (! -d $dir) {
		print STDERR "ERROR: Unable to find directory $dir\n";
		push(@exclude, qr/^$dir/);
		return 0;
	}
	if (! -f "$dir/content.m4") {
		print STDERR "ERROR: Unable to find file $_/content.m4\n";
		push(@exclude, qr/^$dir/);
		return 0;
	}
	if (! -f "$dir/structure.m4") {
		print STDERR "ERROR: Unable to find file $_/structure.m4\n";
		push(@exclude, qr/^$dir/);
		return 0;
	}

	$count = scalar(parse_content $dir);
	# print DEBUG "Got immediate count as $count\n";

	@dirs = parse_structure $dir;
	@dirs = grep { $$_{'type'} =~ /narrow|letter/ } @dirs;
	@dirs = map { $$_{'link'} } @dirs;
	for (@dirs) {
		$count += page_count_build($_);
	}

	$pagecount{$dir} = $count;
	print DEBUG "$count: $dir\n";
	return $count;
}

sub page_count_get {
	my $dir = shift;
	if (!defined($pagecount{$dir})) {
		print STDERR "\$pagecount{$dir} not defined\n";
		return 0;
	}

	return $pagecount{$dir};
}

sub exclude_list_build {
	# @exclude = map { "Top/$_" } @exclude;
	# @exclude = map { qr/$_/ } @exclude;
	my $re = "^Top/(" . join("|", @exclude) . ")";
	@exclude = ( qr/$re/ );
	print "Exclusion: @exclude\n";
}

sub excluded {
	my $link = shift;
	# print DEBUG "Link is $link\n";
	foreach my $re (@exclude) {
		# print "Testing $re\n";
		return 1 if $link =~ $re;
	}
	# print DEBUG "Link OK\n";
	return 0;
}

sub drawhead {
	print OUTPUT << "EOM";
<HTML>
<HEAD>

<TITLE>Hotels, Motels, Lodging, Inns and Resorts :
				www.hotel-search-engine.com</TITLE>

</HEAD>
<BODY BGCOLOR="#FFFFFF" TEXT="#000000" LINK="#434A99"
				VLINK="#434A99" ALINK="#80C080">
<DIV ALIGN=center>

<TABLE WIDTH=600 CELLPADDING=0 CELLSPACING=0 BORDER=0>
<TR><TD BGCOLOR="#535AA2">
	<A HREF="/featured"><IMG WIDTH=50 HEIGHT=20
					SRC="/images/popular.gif"
					BORDER=0 ALIGN=right></A>
	<A HREF="/hotels"><IMG WIDTH=50 HEIGHT=20
					SRC="/images/featured.gif"
					BORDER=0 ALIGN=right></A>
	<A HREF="/"><IMG WIDTH=220 HEIGHT=20
					SRC="/images/logo.gif"
					BORDER=0></A><BR>

</TD></TR><TR><TD BGCOLOR="#80C080">

	<FONT FACE="Arial,Helvetica" SIZE=2>
	<TABLE CELLSPACING=10 CELLPADDING=0 BORDER=0>
		<TR><TD><FONT FACE="Arial,Helvetica" SIZE=2>
		<FORM METHOD=POST ACTION="/apps/datafile/datafile.cgi">
		<INPUT TYPE=HIDDEN NAME="Company" VALUE="oworld">
		<INPUT TYPE=HIDDEN NAME="DataFileTitle" VALUE="oworld">
		<INPUT TYPE=HIDDEN NAME="TemplateTitle" VALUE="oworld">
		Location: <INPUT TYPE=TEXT NAME="Match *" SIZE=15>
		<INPUT TYPE="submit" VALUE="Find Hotels"><BR>
		</FONT></TD></TR></FORM>
	</TABLE>
	</FONT>

</TD></TR><TR><TD BGCOLOR="#FFFFFF" ALIGN=center>
	<FONT FACE="Arial,Helvetica" SIZE=2>
	<TABLE WIDTH=550 CELLSPACING=10 CELLPADDING=0 BORDER=0>
	<TR><TD VALIGN=top COLSPAN=2>
		<FONT FACE="Arial,Helvetica" SIZE=2>
		<B>
EOM
}

# The location block goes in here
sub drawlocation {
	my @location = split(/\//, $_[0]);
	my $path;

	print OUTPUT "\t\t<A HREF=\"$dstdir/Top/\">Home</A>:\n";

	# XXX These parameters just work, OK?
	for (my $i = 1; $i < (@titles - 2); $i++) {
		$path = join("/", @location[0..$i]);
		print OUTPUT "\t\t<A HREF=\"$dstdir/$path\">";
		print OUTPUT "$titles[$i]</A>:\n";
	}
	print OUTPUT "\t\t$titles[-1]\n";
}

sub drawhigh {
	print OUTPUT << "EOM";
		</B>

		&nbsp;<I>($_[0])</I>
		<BR>
		</FONT>
	</TD></TR><TR>
EOM
}

# The link block goes in here
sub drawletterbar {
	my @letters = grep { $$_{'type'} eq 'letter' } @_;

	if ($#letters > 0) {
		my %data = map { $$_{'title'} => $$_{'link'} } @letters;
		@letters = sort keys %data;
		# Was $%data{$_}
		my @out = map { "<A HREF=\"$data{$_}\">$_</A>" } @letters;
		print OUTPUT "\t\t[&nbsp" .  join(" |\n", @out) . "\n&nbsp;]\n";
	}
}

sub drawlink {
	my $data = shift;
	my $type = $$data{'type'};
	my $title = $$data{'title'};
	my $link = $$data{'link'};
	my $item = basename($link);
	my $count = page_count_get($link);

	# Don't cope with related here
	print OUTPUT "\t\t<LI><A HREF=\"$dstdir/$link\"><B>$title</B></A>";
	print OUTPUT "@" if ($type eq 'symbolic');
	print OUTPUT " &nbsp;<I>($count)</I>\n";
}

sub drawlinks {
	@_ = grep { $$_{'type'} =~ /narrow|symbolic/ } @_;
	@_ = sort { $$a{'title'} cmp $$b{'title'} } @_;

	my $i;

	print OUTPUT "\t<TD VALIGN=top>\n";
	print OUTPUT "\t\t<FONT FACE=\"Arial,Helvetica\" SIZE=2>\n";
	print OUTPUT "\t\t<UL>\n";
	for ($i = 0; $i < (@_ / 2); ++$i) {
		drawlink $_[$i];
	}
	print OUTPUT "\t\t</UL>\n";
	print OUTPUT "\t\t</FONT>\n";
	print OUTPUT "\t</TD>\n";

	print OUTPUT "\t<TD VALIGN=top>\n";
	print OUTPUT "\t\t<FONT FACE=\"Arial,Helvetica\" SIZE=2>\n";
	print OUTPUT "\t\t<UL>\n";
	for (; $i < @_; ++$i) {
		drawlink $_[$i];
	}
	print OUTPUT "\t\t</UL>\n";
	print OUTPUT "\t\t</FONT>\n";
	print OUTPUT "\t</TD>\n";
}

sub drawlow {
	print OUTPUT << "EOM";
	</TR><TR><TD BGCOLOR="#80C080" COLSPAN=2>
		<FONT FACE="Arial,Helvetica" SIZE=2>
		<IMG WIDTH=550 HEIGHT=2 SRC="images/trans1x1.gif"><BR>
		</FONT>
	</TD></TR><TR><TD VALIGN=top COLSPAN=2>
		<FONT FACE="Arial,Helvetica" SIZE=2>
		<UL>
EOM
}

# The data block goes in here
sub drawpage {
	my $page = shift;
	print OUTPUT "\t\t<LI><A HREF=\"$$page{'link'}\">";
	print OUTPUT "$$page{'title'}</A> - $$page{'desc'}\n";
}

sub drawpages {
	@_ = sort { $$a{'title'} cmp $$b{'title'} } @_;
	foreach (@_) {
		drawpage $_;
	}
}

sub drawfoot {
	my $dir = shift;
	$dir =~ s^Top/^^;
	print OUTPUT << "EOM";
		</UL>
		</FONT>
	</TD></TR></TABLE>
	</FONT>
</TD></TR><TR><TD BGCOLOR="#80C080">
	<FONT FACE="Arial,Helvetica" SIZE=2>
	<IMG WIDTH=600 HEIGHT=5 SRC="images/trans1x1.gif"><BR>
</TD></TR>
</TABLE>

<P>
<TABLE BORDER="0" BGCOLOR="#336600" CELLPADDING="3" CELLSPACING="0">
<TR><TD>
<TABLE WIDTH="100%" CELLPADDING="2" CELLSPACING="0" BORDER="0">
	<TR ALIGN="CENTER"><TD>
		<FONT FACE="sans-serif, Arial, Helvetica" size="2"color="#FFFFFF">
		Help build the largest human-edited directory on the web.</font>
	</TD></TR><TR BGCOLOR="#CCCCCC" ALIGN="center"><TD>
		<FONT FACE="sans-serif, Arial, Helvetica" SIZE="2">

		<A HREF="http://dmoz.org/cgi-bin/add.cgi?where=$dir">Submit a Site</A> -
		<A HREF="http://dmoz.org/about.html"><b>Open Directory Project</B></A> -
		<A HREF="http://dmoz.org/cgi-bin/apply.cgi?where=$dir">

		Become an Editor</A> </FONT>
	</TD></TR>
	</TABLE>

</TD></TR>
</TABLE>

</DIV>
</BODY>
</HTML>

EOM
}

sub build_array {
	my $args = shift;
	$args =~ s/^<(.*)>$/$1/;
	return split(/>, *</, $args);
}

sub basename {
	my $dir = shift;
	$dir =~ s,^.*/,,;
	return $dir;
}

sub dirname {
	my $dir = shift;
	$dir =~ s,^(.*)/.*$,$1,;
	return $dir;
}

sub parse_content {
	my $dir = shift;
	my @pages = ();
	my @args;

	open(CONTENT, "$dir/content.m4")
					or die "Can't open $dir/content.m4: $!";

	while (<CONTENT>) {
		if (/^_PAGE_PAGE_\((.*)\)$/) {
			@args = build_array($1);
			push(@pages, {
					'title'	=> $args[0],
					'desc'	=> $args[1],
					'link'	=> $args[2]
				});
		}
		elsif (/_TOPIC_(LINK|CATID)_\(.*\)$/) {
		}
		else {
			print STDERR "Unidentified ($.): $_\n";
		}
	}

	close(CONTENT);

	@pages = sort { $$a{'title'} cmp $$b{'title'} } @pages;

	return @pages;
}

sub parse_structure {
	my $dir = shift;
	my(@args, $str);	# Temporaries
	my @links = ();

	open(STRUCTURE, "$dir/structure.m4")
					or die "Can't open $dir/structure.m4: $!";

	# It might go more slowly, but I really shouldn't hash here
	# in case we get a duplicate key. I can always rethink this later
	while (<STRUCTURE>) {
		study;
		if (/^_TOPIC_TITLE_\((.*)\)$/) {
			($title = $1) =~ s/^<(.*)>$/$1/;
		}
		elsif (/^_TOPIC_NARROW_\((.*)\)$/) {
			@args = build_array($1);
			($str = basename($args[0])) =~ s/_/ /g;
			push(@links, {
					'type'	=> 'narrow',
					'title'	=> $str,
					'link'	=> $args[0]
				}) if not excluded $args[0];
		}
		elsif (/_TOPIC_REL_\((.*)\)$/) {
			@args = build_array($1);
			($str = basename($args[0])) =~ s/_/ /g;
			push(@links, {
					'type'	=> 'related',
					'title'	=> $str,
					'link'	=> $args[0]
				}) if not excluded $args[0];
		}
		elsif (/_TOPIC_SYM_\((.*)\)$/) {
			@args = build_array($1);
			($str, $args[0]) = split(/:/, $args[0], 2);
			push(@links, {
					'type'	=> 'symbolic',
					'title'	=> $str,
					'link'	=> $args[0]
				}) if not excluded $args[0];
		}
		elsif (/_TOPIC_LBAR_\((.*)\)$/) {
			@args = build_array($1);
			push(@links, {
					'type'	=> 'letter',
					'title'	=> substr($args[0], -1),
					'link'	=> $args[0]
				}) if not excluded $args[0];
		}
		elsif (/_TOPIC_(NEWS|CATID)_\(.*\)$/) {
		}
		else {
			print STDERR "Unidentified ($.): $_\n";
		}
	}

	close(STRUCTURE);

	return @links;
}

sub parse {
	local $dir = shift;
	print DEBUG "Processing in $dir\n";

	local $title;
	my @pages = parse_content $dir;
	my @links = parse_structure $dir;	# sets $title
	my $count = page_count_get $dir;
	my @subdirs = grep { $$_{'type'} =~ /narrow|letter/ } @links;
	@subdirs = map { $$_{'link'} } @subdirs;

	push(@titles, $title);

	local *OUTPUT;
	open(OUTPUT, ">$dir/index.html")
					or die "Can't open >$dir/index.html: $!";

	drawhead;
	drawlocation $dir;
	drawhigh $count;
	drawletterbar @links;
	drawlinks @links;
	drawlow;
	drawpages @pages;
	drawfoot $dir;

	close(OUTPUT);

	foreach (@subdirs) {
		&parse($_);
	}

	pop(@titles);
	return $count;
}

MAIN:

if (! -d $start) {
	die "Could not find start directory '$start'\n";
}


exclude_list_build;
page_count_build $start;
print DEBUG "Pruning directories:\n\t" . join("\n\t", @exclude) . "\n";
parse $start;
