#include "library.h"
#include "vector.h"
#include "hash.h"
#include "rope.h"
#include "element.h"
#include "smalloc.h"
#include "lexer.h"
#include "parser.h"
#include "symtab.h"

element_t *
element_new(char *word, void *data)
{
	element_t	*t;

	t = (element_t *)smalloc(sizeof(element_t));
	if (word) {
		t->ttype = symtab_get(word);
		t->attr = data;
		t->cdata = NULL;
	}
	else {
		t->ttype = -1;
		t->attr = NULL;
		t->cdata = data;
	}
	t->elem = NULL;
	return t;
}

void
element_delete(element_t *t)
{
	if (t->attr) hash_delete(t->attr);
	if (t->elem) vector_delete(t->elem);
	if (t->cdata) rope_delete(t->cdata);
	sfree(t);
}

int
element_type(element_t *t)
{
	return t->ttype;
}

void
element_attr_set(element_t *t, char *key, rope_t *data)
{
	if (!t->attr)
		t->attr = hash_new((delete_t)rope_delete, 10);
	hash_add(t->attr, key, data);
}

rope_t *
element_attr_get(element_t *t, char *key)
{
	if (!t->attr)
		return NULL;
	return hash_get(t->attr, key);
}

void
element_push_back(element_t *t, element_t *d)
{
	if (!t->elem)
		t->elem = vector_new((delete_t)element_delete, 10);
	vector_push_back(t->elem, d);
}

int
element_elem_length(element_t *t)
{
	if (!t->elem)
		return 0;
	return vector_length(t->elem);
}

element_t *
element_elem_get(element_t *t, int off)
{
	if (!t->elem)
		return NULL;
	return vector_data(t->elem, off);
}

rope_t *
element_cdata(element_t *e)
{
	return e->cdata;
}
