%{
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include "websuite.h"
#include "element.h"
#include "lexer.h"
#include "parser.h"

YYSTYPE yylval;

// static int lineno = 1;

%}

%option nounput
%option noyywrap
%option caseless

%x tag
%x string

LANGLE	<
RANGLE	>

WORD	[[:alnum:]:]+
	/* UNUSED since we now have a string state */
STRING	\"[^\"\n]*\"
	/* A string starting with a non-space, finishing with the same */
CDATA	[^<[:space:]]([^<]*[^<[:space:]])?

SPACE	[ \t\r\f\v]+
NEWLINE	\n

%%

<*>{SPACE}			{ }
<*>{NEWLINE}		{ /* lineno++; */ }

<INITIAL>{LANGLE}	{ BEGIN(tag); return '<'; }
<tag>{RANGLE}		{ BEGIN(INITIAL); return '>'; }

<tag>{WORD}			{
						char	*cp;
						strncpy(yylval.text, yytext, TEXTLEN - 1);
						yylval.text[(TEXTLEN - 1)] = 0;
						for (cp = yylval.text; *cp; cp++)
							*cp = tolower(*cp);

						/* Trim trailing '1' */
						if (*--cp == '1') *cp = 0;

						return T_WORD;
					}

	/* The weird antics here are to make sure we always get a string
	 * even when we have "". So it isn't enough just to return a
	 * rope_new(yytext) in the [^\"]* rule. It would never be
	 * matched. I'm doing this to get strings out of the <tag>
	 * state, so that I can then do funny things to any data in the
	 * tag state. */
<tag>\"				{ BEGIN(string); yylval.rope = rope_new("");	}
<string>\"			{ BEGIN(tag); return T_STRING;					}
<string>[^\"]*		{ rope_add(yylval.rope, yytext);				}

<INITIAL>{CDATA}	{
						yylval.rope = rope_new(yytext);
						return T_CDATA;
					}

<*>.				{ return *yytext; }

%%

const char *tokname[] = {
	[T_WORD]	= "WORD",
	[T_STRING]	= "STRING",
	[T_CDATA]	= "CDATA"
};

int
zzlex()
{
	int	 i;
	i = yylex();
	printf("Lexer returned %d ", i);
	if (i < 127)
		printf("(%c)\n", i);
	else
		printf("(%s)\n", tokname[i]);

	if (i == T_CDATA) {
		printf("CDATA is \"%s\"\n", rope_chars(yylval.rope));
	}
	else if (i == T_WORD) {
		printf("WORD is \"%s\"\n", yylval.text);
	}
	else if (i == T_STRING) {
		printf("STRING is \"%s\"\n", rope_chars(yylval.rope));
	}
	return i;
}
