#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sysexits.h>
#include <errno.h>
#include "library.h"
#include "rope.h"
#include "output.h"

extern FILE *debug;

static int		 _type = 0;
static FILE		*_output = NULL;
static rope_t	*_cwd = NULL;
static rope_t	*_filename = NULL;

static void
output_init(void)
{
	char	 buf[BUFSIZ];

	if (!_type) {
		fprintf(stderr, "Cannot output without a known output type\n");
		exit(EX_USAGE);
	}

	if (!getcwd(buf, BUFSIZ)) {
		fprintf(stderr, "Cannot get current working directory - "
						"path too long\n");
		exit(1);
	}
	_cwd = rope_new(buf);
	rope_add(_cwd, "/");

	_filename = rope_new("");
}

bool
output_set(char *name)
{
	if (!_filename)
		output_init();

	if (_output)
		fclose(_output);

	rope_set(_filename, rope_chars(_cwd));
	rope_add(_filename, name);
	if (mkdir(rope_chars(_filename), S_IRWXU) < 0) {
		if (errno != EEXIST) {
			fprintf(stderr, "Could not make target directory %s:\n",
							rope_chars(_filename));
			perror("mkdir");
			exit(EX_CANTCREAT);
		}
	}

	if (_type == OUTPUT_CONTENT)
		rope_add(_filename, "/content.m4");
	else	/* This is safe because we checked on the way in */
		rope_add(_filename, "/structure.m4");

	// fprintf(debug, "Opening %s\n", rope_chars(_filename));
	_output = fopen(rope_chars(_filename), "w");
	if (!_output) {
		fprintf(stderr, "Could not fopen output file %s:\n",
						rope_chars(_filename));
		perror("fopen");
		exit(EX_CANTCREAT);
	}

	return TRUE;
}

bool
output_type_set(int type)
{
	switch(type) {
		case OUTPUT_CONTENT:
		case OUTPUT_STRUCTURE:
			_type = type;
			break;
		default:
			fprintf(stderr, "Output type %d unknown\n", type);
			exit(EX_USAGE);
	}
	return TRUE;
}

int
output_type_get(void)
{
	return _type;
}

FILE *
output_stream(void)
{
	return _output;
}
