%{
#include <stdio.h>
#include <unistd.h>
#include <getopt.h>
#include <sysexits.h>
#include "websuite.h"
#include "element.h"
#include "symtab.h"
#include "lexer.h"
#include "output.h"

/* This is a huge, monolithic file. Huge monolithic files are Bad(TM).
 * Tell me about it.
 * S.
 */


// #define yylex zzlex
#define CACHE_TOKENS

rope_t	*c_title = NULL;
rope_t	*c_desc = NULL;
rope_t	*c_pri = NULL;
FILE	*null = NULL;
FILE	*output = NULL;
FILE	*debug = NULL;
FILE	*error = NULL;

/* This makes them modifiable in data space */
static char KEY_TOPIC[] = "topic";
static char KEY_LINK[] = "link";

static char KEY_PAGE[] = "externalpage";
static char KEY_TITLE[] = "d:title";
static char KEY_DESC[] = "d:description";

static char KEY_CATID[] = "catid";
static char KEY_PRI[] = "priority";

static char	KEY_NARROW[] = "narrow";
static char	KEY_SYM[] = "symbolic";
static char KEY_NEWS[] = "newsgroup";
static char KEY_REL[] = "related";
static char KEY_LBAR[] = "letterbar";

static char KEY_UPD[] = "lastupdate";
static char KEY_EDIT[] = "editor";

static char KEY_ALIAS[] = "alias";
// static char KEY_TARG[] = "target";	/* I drop aliases */


#ifdef CACHE_TOKENS

static int ttype_topic;
static int ttype_link;
static int ttype_page;
static int ttype_title;
static int ttype_desc;
static int ttype_catid;
static int ttype_pri;
static int ttype_narrow;
static int ttype_sym;
static int ttype_news;
static int ttype_rel;
static int ttype_upd;
static int ttype_edit;

#define TYPE_TOPIC	ttype_topic
#define TYPE_LINK	ttype_link
#define TYPE_PAGE	ttype_page
#define TYPE_TITLE	ttype_title
#define TYPE_DESC	ttype_desc
#define TYPE_CATID	ttype_catid
#define TYPE_PRI	ttype_pri
#define TYPE_NARROW	ttype_narrow
#define TYPE_SYM	ttype_sym
#define TYPE_NEWS	ttype_news
#define TYPE_REL	ttype_rel
#define TYPE_UPD	ttype_upd
#define TYPE_EDIT	ttype_edit

#else

#define TYPE_TOPIC	symtab_get(KEY_TOPIC)
#define TYPE_LINK	symtab_get(KEY_LINK)
#define TYPE_LINK1	symtab_get(KEY_LINK1)
#define TYPE_PAGE	symtab_get(KEY_PAGE)
#define TYPE_TITLE	symtab_get(KEY_TITLE)
#define TYPE_DESC	symtab_get(KEY_DESC)
#define TYPE_CATID	symtab_get(KEY_CATID)
#define TYPE_PRI	symtab_get(KEY_PRI)
#define TYPE_NARROW	symtab_get(KEY_NARROW)
#define TYPE_SYM	symtab_get(KEY_SYM)
#define TYPE_NEWS	symtab_get(KEY_NEWS)
#define TYPE_REL	symtab_get(KEY_REL)
#define TYPE_UPD	symtab_get(KEY_UPD)
#define TYPE_EDIT	symtab_get(KEY_EDIT)

#endif

#define TYPE_ALIAS	symtab_get(KEY_ALIAS)
// #define TYPE_TARG	symtab_get(KEY_TARG)
#define TYPE_LBAR	symtab_get(KEY_LBAR)

int yylex();

static inline element_t * element_complete_process(element_t *e);

void
yyerror(char *str)
{
	fprintf(stderr, "yyerror: %s", str);
}

%}

%union {
	int			 ttype;
	char		 text[TEXTLEN];
	rope_t		*rope;
	hash_t		*attr;
	element_t	*elem;
}

	/* %start rdf_tag */
%start element

%token <ttype> T_RDF

%token <ttype> T_TOPIC
%token <ttype> T_PAGE

%token <ttype> T_TITLE
%token <ttype> T_DESC
%token <ttype> T_CATID

%token <ttype> T_LINK

%token <ttype> T_PRI

%token <text> T_WORD
%token <rope> T_STRING
%token <rope> T_CDATA

%type <attr> attribute_list;

%type <ttype> tag_end;
%type <elem> tag_start tag_single tag_complete
%type <elem> element

%%

	/* General declarations */

attribute_list	:
				{
					// fprintf(debug, "Made attribute list\n");
					$$ = NULL;
				}
				| T_WORD '=' T_STRING attribute_list
				{
					if ($4)
						$$ = $4;
					else
						$$ = hash_new((delete_t)rope_delete, 10);
					hash_add($$, $1, $3);
					/*
					fprintf(debug, "Added attribute %s = %s\n",
									$1, rope_chars($3));
					*/
				}
				;

tag_start		: '<' T_WORD attribute_list '>'
				{
					// fprintf(debug, "Start tag %s\n", $2);
					$$ = element_new($2, $3);
				}
				| tag_start element
				{
					$$ = $1;
					/* The tag might have been destructed */
					if ($2)
						element_push_back($$, $2);
					// if (!$2) fprintf(debug, "Hit destructed tag\n");
				}
				;

tag_end			: '<' '/' T_WORD '>'
				{
					// fprintf(debug, "End tag %s\n", $3);
					$$ = symtab_get($3);
				}
				;

tag_single		: '<' T_WORD attribute_list '/' '>'
				{
					// fprintf(debug, "Singleton tag %s\n", $2);
					$$ = element_new($2, $3);
				}
				;

tag_complete	: tag_single
				{
					$$ = $1;
				}
				| tag_start tag_end
				{
					if (element_type($1) != $2) {
						fprintf(error,"Tag type %d does not match %d\n",
										element_type($1), $2);
						yyerror("Tag stack error");
						exit(0);
					}

					/* Since all the tags we are interested in have
					 * a start and an end, we can do our special
					 * parsing here, using $2 as our tag type */

					/* Might return NULL */
					$$ = element_complete_process($1);
				}
				;

element			: T_CDATA
				{
					$$ = element_new(NULL, yylval.rope);
					// fprintf(debug, "Made CDATA element\n");
				}
				| tag_complete
				{
					// fprintf(debug, "Made TAG element\n");
					/* This MIGHT be NULL for a destructed tag */
					$$ = $1;
				}
				;

%%

/* We have <x>cdata</x>, we want the cdata */
static inline rope_t *
parser_element_cdata(element_t *e)
{
	element_t	*d;
	rope_t		*r;
	int			 len;
	int			 i;

	len = element_elem_length(e);
	for (i = 0; i < len; i++) {
		d = element_elem_get(e, i);
		/* XXX Here we rely on that it is not a union, because
		 * we don't bother checking element_type() */
		r = element_cdata(d);
		if (r) return r;
	}

	return NULL;
}

static inline void
parser_topic(element_t *e)
{
	element_t	*d;
	rope_t		*r;
	int			 dtype;
	int			 len;
	int			 i;

	r = element_attr_get(e, "r:id");
	if (!r) {
		fprintf(error, "No 'r:id' in Topic tag\n");
		return;
	}
	else {
		fprintf(debug, "Opening file %s\n", rope_chars(r));
		/* Fatal if error */
		output_set(rope_chars(r));
		output = output_stream();
	}

	len = element_elem_length(e);
	for (i = 0; i < len; i++) {
		d = element_elem_get(e, i);
		/* If I made element_type() a macro, it would be faster */
		dtype = element_type(d);

		if (dtype == TYPE_LINK) {
			r = element_attr_get(d, "r:resource");
			if (r) {
				fprintf(output, "_TOPIC_LINK_(<%s>)\n", rope_chars(r));
			}
			else {
				fprintf(stderr, "No resource in link element\n");
			}
		}
		else if (dtype == TYPE_CATID) {
			/* Pull the catid */
			r = parser_element_cdata(d);
			if (r)
				fprintf(output, "_TOPIC_CATID_(<%s>)\n", rope_chars(r));
			else
				fprintf(stderr, "No CDATA in catid element\n");
		}
		else if (dtype == TYPE_TITLE) {
			r = parser_element_cdata(d);
			if (r)
				fprintf(output, "_TOPIC_TITLE_(<%s>)\n", rope_chars(r));
			else
				fprintf(stderr, "No CDATA in Title element\n");
		}

		/* Structure stuff */
		else if (dtype == TYPE_NARROW) {
			r = element_attr_get(d, "r:resource");
			if (r)
				fprintf(output, "_TOPIC_NARROW_(<%s>)\n",rope_chars(r));
			else
				fprintf(stderr, "No resource in narrow element\n");
		}
		else if (dtype == TYPE_SYM) {
			r = element_attr_get(d, "r:resource");
			if (r)
				fprintf(output, "_TOPIC_SYM_(<%s>)\n", rope_chars(r));
			else
				fprintf(stderr, "No resource in symbolic element\n");
		}
		else if (dtype == TYPE_NEWS) {
			r = element_attr_get(d, "r:resource");
			if (r)
				fprintf(output, "_TOPIC_NEWS_(<%s>)\n", rope_chars(r));
			else
				fprintf(stderr, "No resource in newsgroup element\n");
		}
		else if (dtype == TYPE_REL) {
			r = element_attr_get(d, "r:resource");
			if (r)
				fprintf(output, "_TOPIC_REL_(<%s>)\n", rope_chars(r));
			else
				fprintf(stderr, "No resource in related element\n");
		}
		else if (dtype == TYPE_LBAR) {
			r = element_attr_get(d, "r:resource");
			if (r)
				fprintf(output, "_TOPIC_LBAR_(<%s>)\n", rope_chars(r));
			else
				fprintf(stderr, "No resource in related element\n");
		}
		else if (dtype == TYPE_UPD) {
		}
		else if (dtype == TYPE_EDIT) {
		}

		else {
			fprintf(stderr, "Unrecognised element in TOPIC: %d\n",
							dtype);
		}
	}
}

static inline void
parser_page(element_t *e)
{
	rope_t		*link = NULL;
	rope_t		*title = NULL;
	rope_t		*desc = NULL;

	element_t	*d;
	int			 dtype;
	int			 len;
	int			 i;

	link = element_attr_get(e, "about");

	len = element_elem_length(e);
	for (i = 0; i < len; i++) {
		d = element_elem_get(e, i);
		dtype = element_type(d);
		if (dtype == TYPE_TITLE)
			title = parser_element_cdata(d);
		else if (dtype == TYPE_DESC)
			desc = parser_element_cdata(d);
		else if (dtype == TYPE_PRI)
			;
		else
			fprintf(stderr, "Unknown page dtype: %d\n", dtype);
	}

	fprintf(output, "_PAGE_PAGE_(<%s>, <%s>, <%s>)\n",
					title?rope_chars(title):"No title",
					desc?rope_chars(desc):"No description",
					link?rope_chars(link):"ERROR");
}

static inline element_t *
element_complete_process(element_t *e)
{
	/* Cache the values */
	int			 etype;

	etype = element_type(e);
	if (etype == TYPE_TOPIC) {
		parser_topic(e);
		element_delete(e);
		e = NULL;
	}
	else if (etype == TYPE_PAGE) {
		parser_page(e);
		element_delete(e);
		e = NULL;
	}
	else if (etype == TYPE_ALIAS) {
		element_delete(e);
		e = NULL;
	}

	return e;
}

#ifdef CACHE_TOKENS
static void
parser_cache_tokens()
{
/* This makes them modifiable in data space */
	ttype_topic = symtab_get(KEY_TOPIC);
	ttype_link = symtab_get(KEY_LINK);
	ttype_page = symtab_get(KEY_PAGE);
	ttype_title = symtab_get(KEY_TITLE);
	ttype_desc = symtab_get(KEY_DESC);
	ttype_catid = symtab_get(KEY_CATID);
	ttype_pri = symtab_get(KEY_PRI);
	ttype_narrow = symtab_get(KEY_NARROW);
	ttype_sym = symtab_get(KEY_SYM);
	ttype_news = symtab_get(KEY_NEWS);
	ttype_rel = symtab_get(KEY_REL);
	ttype_upd = symtab_get(KEY_UPD);
	ttype_edit = symtab_get(KEY_EDIT);
}
#endif

void
usage(void)
{
	printf("Usage: rdfparser -s | -d\n");
	printf("\t-c:\tParse RDF content file\n");
	printf("\t-s:\tParse RDF structure file\n");
}

int
main(int argc, char *argv[])
{
	int	 c;

	library_init("rdfparser");

	while ((c = getopt(argc, argv, "cs")) != EOF) {
		switch(c) {
			case 'c':
				output_type_set(OUTPUT_CONTENT);
				break;
			case 's':
				output_type_set(OUTPUT_STRUCTURE);
				break;
			default:
				fprintf(stderr, "Unrecognised option %c\n", c);
				usage();
				exit(EX_USAGE);
		}
	}
	argc -= optind;
	argv += optind;

	if (!output_type_get()) {
		fprintf(stderr, "No output type specified\n");
		usage();
		exit(EX_USAGE);
	}

	fprintf(stderr, "Output streams...\n");
	null = fopen("/dev/null", "w");
	if (!null) {
		perror("fopen /dev/null");
		return 1;
	}
	output = stdout;
	debug = stdout;
	error = stderr;

	fprintf(stderr, "Symbol table...\n");
	symtab_create();

#ifdef CACHE_TOKENS
	fprintf(stderr, "Table cache overlay...\n");
	parser_cache_tokens();
#endif

	yyparse();

	return 0;
}
