%{
/*
 * Solitaire CGI - template parser
 * Copyright (C) 1999 Open World Ltd
 * Written by Shevek <shevek@anarres.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include "library.h"
#include "config.h"
#include "log.h"
#include "hash.h"
#include "smalloc.h"
#include "cgi.h"

#define STATE_SIZE	(13)

#define HOLE_EMPTY	(0)
#define HOLE_FULL	(1)
#define HOLE_NONE	(2)

#define MOVE_UP		(1)
#define MOVE_LEFT	(2)
#define MOVE_RIGHT	(4)
#define MOVE_DOWN	(8)

typedef unsigned char hole_t;

typedef
struct _state_t {
	hole_t	**board;
	char	*instance;
	char	*script;
	FILE	*input;
} state_t;

static void template_print(int y, int x);

state_t	state;

%}

%option nounput
%option noyywrap

%%

	/* Hardcode the references */

\{SquareA0\}	template_print(2, 5);
\{SquareA1\}	template_print(2, 6);
\{SquareA2\}	template_print(2, 7);
\{SquareA3\}	template_print(3, 5);
\{SquareA4\}	template_print(3, 6);
\{SquareA5\}	template_print(3, 7);
\{SquareA6\}	template_print(4, 5);
\{SquareA7\}	template_print(4, 6);
\{SquareA8\}	template_print(4, 7);

\{SquareB0\}	template_print(5, 2);
\{SquareB1\}	template_print(5, 3);
\{SquareB2\}	template_print(5, 4);
\{SquareB3\}	template_print(6, 2);
\{SquareB4\}	template_print(6, 3);
\{SquareB5\}	template_print(6, 4);
\{SquareB6\}	template_print(7, 2);
\{SquareB7\}	template_print(7, 3);
\{SquareB8\}	template_print(7, 4);

\{SquareC0\}	template_print(5, 5);
\{SquareC1\}	template_print(5, 6);
\{SquareC2\}	template_print(5, 7);
\{SquareC3\}	template_print(6, 5);
\{SquareC4\}	template_print(6, 6);
\{SquareC5\}	template_print(6, 7);
\{SquareC6\}	template_print(7, 5);
\{SquareC7\}	template_print(7, 6);
\{SquareC8\}	template_print(7, 7);

\{SquareD0\}	template_print(5, 8);
\{SquareD1\}	template_print(5, 9);
\{SquareD2\}	template_print(5, 10);
\{SquareD3\}	template_print(6, 8);
\{SquareD4\}	template_print(6, 9);
\{SquareD5\}	template_print(6, 10);
\{SquareD6\}	template_print(7, 8);
\{SquareD7\}	template_print(7, 9);
\{SquareD8\}	template_print(7, 10);

\{SquareE0\}	template_print(8, 5);
\{SquareE1\}	template_print(8, 6);
\{SquareE2\}	template_print(8, 7);
\{SquareE3\}	template_print(9, 5);
\{SquareE4\}	template_print(9, 6);
\{SquareE5\}	template_print(9, 7);
\{SquareE6\}	template_print(10, 5);
\{SquareE7\}	template_print(10, 6);
\{SquareE8\}	template_print(10, 7);

%%

/* Start staty stuff */

static void
state_build_part(hole_t **s, char *temp, int y, int x)
{
	int	 val;
	int	 b;
	int	 i;
	int	 j;

	if (temp)
		val = atol(temp);
	else
		val = 0;

	b = 1;
	for (j = y; j < (y + 3); j++) {
		for (i = x; i < (x + 3); i++) {
			s[j][i] = (val & b) ? HOLE_FULL : HOLE_EMPTY;
			b <<= 1;
		}
	}
}

static void
state_build(hash_t *cgihash, hole_t **s)
{
	int		 i;
	int		 j;

	for (i = 0; i < STATE_SIZE; i++) {
		for (j = 0; j < STATE_SIZE; j++) {
			s[i][j] = HOLE_NONE;
		}
	}

	state_build_part(s, hash_get(cgihash, "a"), 2, 5);
	state_build_part(s, hash_get(cgihash, "b"), 5, 2);
	state_build_part(s, hash_get(cgihash, "c"), 5, 5);
	state_build_part(s, hash_get(cgihash, "d"), 5, 8);
	state_build_part(s, hash_get(cgihash, "e"), 8, 5);
}

int
state_moves(hole_t **s, int y, int x)
{
	int	 moves = 0;

	if (s[y][x] == HOLE_EMPTY)
		return 0;

	if (s[(y)][(x + 1)] == HOLE_FULL)
		if (s[(y)][(x + 2)] == HOLE_EMPTY)
			moves |= MOVE_RIGHT;
	if (s[(y)][(x - 1)] == HOLE_FULL)
		if (s[(y)][(x - 2)] == HOLE_EMPTY)
			moves |= MOVE_LEFT;
	if (s[(y + 1)][(x)] == HOLE_FULL)
		if (s[(y + 2)][(x)] == HOLE_EMPTY)
			moves |= MOVE_DOWN;
	if (s[(y - 1)][(x)] == HOLE_FULL)
		if (s[(y - 2)][(x)] == HOLE_EMPTY)
			moves |= MOVE_UP;

	return moves;
}

static int
state_unbuild_part(hole_t **s, int y, int x)
{
	int	 ret = 0;
	int	 i, j;
	int	 b;

	b = 1;
	for (j = y; j < (y + 3); j++) {
		for (i = x; i < (x + 3); i++) {
			if (s[j][i] == HOLE_FULL)
				ret |= b;
			b <<= 1;
		}
	}
	return ret;
}

/* End staty stuff */

/* Start lexy stuff */

#define OLD_YPOS	(y)
#define OLD_XPOS	(x)
#define NEW_YPOS	(y + 2 * foo[dir][0])
#define NEW_XPOS	(x + 2 * foo[dir][1])
#define DEL_YPOS	(y + foo[dir][0])
#define DEL_XPOS	(x + foo[dir][1])

static inline void
template_print_image(int y, int x, int moves, int dir, char m, char d)
{
	/* Watch carefully... */
	const int foo[9][2] = {
		[MOVE_UP]		= { -1,  0 },
		[MOVE_LEFT]		= {  0, -1 },
		[MOVE_DOWN]		= {  1,  0 },
		[MOVE_RIGHT]	= {  0,  1 }
	};

	if (moves & dir) {
		state.board[OLD_YPOS][OLD_XPOS] = HOLE_EMPTY;
		state.board[DEL_YPOS][DEL_XPOS] = HOLE_EMPTY;
		state.board[NEW_YPOS][NEW_XPOS] = HOLE_FULL;
		fprintf(yyout, "<A HREF=\"%s?case=%s&a=%d&b=%d&c=%d&d=%d&e=%d"
						"\">",
						state.script, state.instance,
						state_unbuild_part(state.board, 2, 5),
						state_unbuild_part(state.board, 5, 2),
						state_unbuild_part(state.board, 5, 5),
						state_unbuild_part(state.board, 5, 8),
						state_unbuild_part(state.board, 8, 5)
						);
	}
	fprintf(yyout, "<IMG SRC=\"" DIR_RESOURCE
					"/solitaire/%s/images/%c.gif\" "
					"HSPACE=0 VSPACE=0 BORDER=0>",
					state.instance, (moves & dir) ? m : d);
	if (moves & dir) {
		state.board[OLD_YPOS][OLD_XPOS] = HOLE_FULL;
		state.board[DEL_YPOS][DEL_XPOS] = HOLE_FULL;
		state.board[NEW_YPOS][NEW_XPOS] = HOLE_EMPTY;
		fprintf(yyout, "</A>");
	}
}

static void
template_print(int y, int x)
{
	int	 moves;
	moves = state_moves(state.board, y, x);

	fprintf(yyout, "<TABLE BORDER=0><TR>\n");

	fprintf(yyout, "<TD></TD>\n");
	fprintf(yyout, "<TD>");
	template_print_image(y, x, moves, MOVE_UP, 'u', 'h');
	fprintf(yyout, "</TD>");
	fprintf(yyout, "<TD></TD>\n");

	fprintf(yyout, "</TR><TR>\n");

	fprintf(yyout, "<TD>");
	template_print_image(y, x, moves, MOVE_LEFT, 'l', 'v');
	fprintf(yyout, "</TD>");
	fprintf(yyout, "<TD>");

	if (state.board[y][x] == HOLE_FULL) {
		fprintf(yyout, "<IMG SRC=\"" DIR_RESOURCE
						"/solitaire/%s/images/c.gif\" "
						"HSPACE=0 VSPACE=0 BORDER=0>",
						state.instance);
	}

#if 0
	if (state.board[y][x] == HOLE_FULL)
		fprintf(yyout, "<FONT COLOR=\"#FF0000\">");
	fprintf(yyout, "[%x]", moves);
	if (state.board[y][x] == HOLE_FULL)
		fprintf(yyout, "</FONT>");
#endif

	fprintf(yyout, "</TD>\n");

	fprintf(yyout, "<TD>");
	template_print_image(y, x, moves, MOVE_RIGHT, 'r', 'v');
	fprintf(yyout, "</TD>");

	fprintf(yyout, "</TR><TR>\n");

	fprintf(yyout, "<TD></TD>\n");
	fprintf(yyout, "<TD>");
	template_print_image(y, x, moves, MOVE_DOWN, 'd', 'h');
	fprintf(yyout, "</TD>");
	fprintf(yyout, "<TD></TD>\n");

	fprintf(yyout, "</TR></TABLE>\n");

}

void
template_init(hash_t *cgihash, hole_t **s)
{
	char	*temp;
	FILE	*input;
	rope_t	*rope;

	temp = getenv(ENV_SCRIPT);
	if (!temp) {
		log("cgisys", PRI_WARNING, "Script could not getenv(\""
						ENV_SCRIPT "\") for base of URL - guessing");
		temp = "/cgi-bin/solitaire.cgi";
	}
	state.script = temp;

	temp = hash_get(cgihash, "case");
	if (!temp) {
		log("cgisys", PRI_WARNING, "No 'case' in hash table - "
						"defaulting");
		temp = "default";
	}
	state.instance = temp;

	state.board = s;

	temp = getenv(ENV_HOSTNAME);
	if (!temp) {
		log("cgisys", PRI_WARNING, "Could not getenv(\""
						ENV_HOSTNAME "\")");
		exit(0);
	}
	rope = rope_new(DIR_DATA "/");
	rope_add(rope, temp);
	rope_add(rope, "/solitaire/");
	rope_add(rope, state.instance);
	rope_add(rope, "/board.tpt");

	// fprintf(stderr, "Rope is %s\n", rope_chars(rope));

	input = fopen(rope_chars(rope), "r");
	if (!input) {
		log("cgisys", PRI_WARNING, "Could not getenv(\""
						ENV_HOSTNAME "\")");
		exit(0);
	}
	
	state.input = input;
	yyin = input;
}

void
template_parse(hash_t *cgihash, hole_t **s)
{
	template_init(cgihash, s);

	printf("Content-type: text/html\n\n");
	yylex();
}

/* End lexy stuff */

/* Start main stuff */

int
main(void)
{
	hash_t	*cgihash;
	hole_t	*state[STATE_SIZE];
	int		 i;

	library_init("solitaire");

	for (i = 0; i < STATE_SIZE; i++)
		state[i] = smalloc(STATE_SIZE * sizeof(hole_t));

	cgihash = cgi_hash_new();
	state_build(cgihash, state);

	template_parse(cgihash, state);

	return 0;
}
