/*
 * Virtual Card CGI - sendmail I/O routines
 * Copyright (C) 1999 Open World Ltd
 * Written by Shevek <shevek@anarres.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "library.h"
#include "hash.h"
#include "log.h"

/* x[0] is an effective (strlen(x) == 0) */
#define SRC_DOMAIN		\
	VCARD_SRC_HOST[0] ?	\
		VCARD_SRC_HOST:	\
		((char *)hash_get(cgihash, ENV_HOSTNAME))

bool
email_send(hash_t *cgihash)
{
	char	*t_email;
	FILE	*output;

	t_email = hash_get(cgihash, "t_email");
	if (!t_email) {
		log("cgiconf", PRI_WARNING, "No 't_email' in CGI hash table - "
						"cannot send email");
		return FALSE;
	}

	output = popen(BIN_SENDMAIL " -t", "w");
	if (!output) {
		log("cgisys", PRI_CRIT, "Could not popen(" BIN_SENDMAIL " -t) "
						": %s", strerror(errno));
		return FALSE;	/* Don't exit */
	}

	/* These are not necessarily constant */
	fprintf(output, "From: %s@%s\n", VCARD_SRC_USER, SRC_DOMAIN);
	fprintf(output, "To: %s\n", t_email);
	fprintf(output, "Subject: V-Card notification\n");
	fprintf(output, "\n");
	fprintf(output, "In Xanadu did Kubla Khan,\n");
	fprintf(output, "A stately greetings card decree.\n");
	fprintf(output, "Where Alph, the sacred packet ran\n");
	fprintf(output, "Through networks measureless to man\n");
	fprintf(output, "\t\tDown to a sunless sea\n");
	fprintf(output, "\n");
	fprintf(output, "A Virtual Greetings Card awaits you at\n");
	fprintf(output, "http://%s/cgi-bin/vcard.cgi?case=%s&card=%s\n",
					(char *)hash_get(cgihash, ENV_HOSTNAME),
					(char *)hash_get(cgihash, "case"),
					(char *)hash_get(cgihash, "card"));
	fprintf(output, "\n");
	fprintf(output, "--\n");
	fprintf(output, "Virtual Greeting Card Server\n");
	fprintf(output, "Administrator " ADM_EMAIL "\n");
	fprintf(output, "Poetry by Coleridge, munged by Shevek\n");
	fprintf(output, "\n");

	pclose(output);

	return TRUE;
}
