#include <stdio.h>

#include "library.h"
#include "vector.h"

#define CNUMLEN	(3)
#define DATELEN (10)
#define DATEOFF	(3)
#define DAYOFF	(19)
#define NIGHTOFF	(27)

const char *wcodes[] = {
   [65] = "Fair",
   [66] = "Partly Cloudy",
   [67] = "Cloudy",
   [68] = "Dust",
   [69] = "Mostly Cloudy",
   [70] = "Foggy",
   [71] = "Hot",
   [72] = "Hazy",
   [73] = "Very Cold",
   [74] = "Snow Shower",
   [75] = "Smoke",
   [76] = "Drizzle",
   [77] = "Flurries",
   [78] = "Windy",
   [79] = "Mixed",
   [80] = "Blizzard",
   [81] = "Blowing Snow",
   [82] = "Rain",
   [83] = "Snow",
   [84] = "Thunderstorm",
   [85] = "Sunny",
   [87] = "Showers",
   [88] = "Sleet",
   [89] = "Freezing Rain",
   [90] = "Freezing Drizzle",
};

void
input_process(char *line)
{
	int		 val;
	char	*daytext;
	char	*nighttext;

	val = atol(&line[DAYOFF]);
	daytext = wcodes[val];

	val = atol(&line[NIGHTOFF]);
	nighttext = wcodes[val];

	printf("Day: %s\nNight: %s\n", daytext, nighttext);
}

bool
input_parse(FILE *input, char *code)
{
	vector_t	*dv;
	char		 line[BUFSIZ];
	int			 numcities;
	char		 citynum[CNUMLEN + 1];
	char		 datebuf[DATELEN + 1];
	time_t		 timebuf;
	struct tm	*tm;

	if (!fgets(line, BUFSIZ, input)) {
		log("cgisys", PRI_CRIT, "Unexpected EOF at start of file");
		return FALSE;
	}

	numcities = atol(line);

	while (numcities--) {
		if (!fgets(line, BUFSIZ, input)) {
			log("cgisys", PRI_CRIT, "Unexpected EOF in cities");
			return FALSE;
		}
		if (strlen(line) < 22) {
			log("cgisys", PRI_CRIT, "Unexpected short line '%s'", line);
			return FALSE;
		}
		if (strncmp(&line[19], code, 2) == 0) {
// DPRINTF((stderr, "Got city %s from line '%s'\n", code, line));
			strncpy(citynum, line, CNUMLEN);
			citynum[CNUMLEN] = 0;
		}
	}

	while (strlen(line)) {
		if (!fgets(line, BUFSIZ, input)) {
			log("cgisys", PRI_CRIT, "Unexpected EOF before data");
			return FALSE;
		}
	}

	time(&timebuf);	/* Offset as necessary by n * 86400 */
	tm = gmtime(&timebuf);
	snprintf(datebuf, DATELEN + 1, "%.4d-$.2d-%.2d",
					tm->tm_year + 1900, tm->tm_mon, tm->tm_mday);

	while (fgets(line, BUFSIZ, input)) {
		if (strncmp(line, citynum, CNUMLEN) == 0) {
			if (strncmp(&line[DATEOFF], datebuf, DATELEN) == 0) {
				input_process(line);
				return TRUE;
			}
		}
	}

	/* We got no weather */
	printf("No data for given city and date\n");
}
